<!doctype>
<head>
  <link rel="stylesheet" href="rickshaw.min.css">
	<script src="http://d3js.org/d3.v3.min.js" charset="utf-8"></script>
  <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.8.1/jquery.min.js"></script>
	<script src="rickshaw.min.js"></script>

<style>
body {
  font: 10px sans-serif;
}
#chart {
    float: left;
    margin-top: 70px;
          }
#y_axis {
   float: left;
   margin-top: 70px;
          }
#legend {
        float: left;
        margin-left: 15px;
        margin-top: 70px;
}
</style>
</head>
<body>

        <div id="y_axis"></div>
        <div id="chart"></div>
        <div id="legend"></div>


<script>

{{ palette }}

d3.json('{{ data_path }}', function(error, json) {

{{ graph }}

{{ x_axis }}

{{ x_axis_num }}

{{ y_axis }}

{{ hover }}

{{ legend }}

graph.render();

});


</script>

</body>