#chart_container {
    position: relative;
    font-family: Arial, Helvetica, sans-serif;
          }
#chart {
    position: relative;
    left: 40px;
          }
#y_axis {
   position: absolute;
   top: 0;
   bottom: 0;
   width: 40px;
          }